#ifdef GL_OES_standard_derivatives
#extension GL_OES_standard_derivatives : enable
#endif

precision highp float;

// our texture
uniform sampler2D u_image[5];
uniform mediump float u_hitTest;
uniform vec2 u_resolution;
uniform float u_shadowType;
uniform float u_zoom;

// the texCoords passed in from the vertex shader.
varying vec4 v_texCoord;
// colour of flat shaded triangles
varying vec4 v_colour;
// shadowcanvas colour mapping
varying vec4 v_hitTestColour;
// which texture to use
varying vec2 v_textureBank;
// circle border width
varying float v_width;
varying float v_circleRadius;
// the size of a pixel in screenspace clip coordinates
varying float v_pixel;
varying vec2 v_donutSegment;

const float PI = 3.1415926535897932384626433832795;
// Performance antialiasing using standard derivatives
// Smooths the fragment over 1 pixel when value is more than threshold
// esentially a better version of step and smoothstep
float aastep(float threshold, float value) {
  float afwidth = length(vec2(dFdx(value), dFdy(value))) * 0.70710678118654757;
  return smoothstep(threshold-afwidth, threshold+afwidth, value);
}

// ensures that numbers are exactly integers 
int roundToInt(float a) {
 return int(floor(a + 0.5));
}

// distance to edge from a triangle, used for smoothing edges of arrows
float edgeFactor(vec3 barycentricCoords){
  vec3 d = fwidth(barycentricCoords);
  vec3 a3 = smoothstep(vec3(0.0), d*3.0, barycentricCoords);
 return min(min(a3.x, a3.y), a3.z);
}

// Read the texture value of a glyph from the specified colour channel
float readColourComponentFromTexture(float textureBank, vec4 texCoord) {
  float texComponent;
  int texBank = roundToInt(textureBank);
  if(texBank == 0) {
    texComponent = texCoord.r;
  } else if(texBank == 1) {
    texComponent = texCoord.g;
  } else {
    texComponent = texCoord.b;
  }
  return texComponent;
}

// Used to reconstruct a glyph from the colourComponent texture
// Read one specific colour only and treat it as an alpha value, setting its colour using colourModifier
vec4 getColourFromComponent(float colourComponent, vec4 colourModifier) {
  float colourAvg = (colourModifier.r + colourModifier.b + colourModifier.g) / 3.0;
  float isWhite = step(1.0,colourAvg);
  // If the text is white boost the alpha to increase contrast
  float whiteTextBoost = isWhite * (step(0.25,colourComponent)*min(colourAvg,0.25));
  float a = colourComponent  + whiteTextBoost;
  float r = colourModifier.r * a;
  float g = colourModifier.g * a;
  float b = colourModifier.b * a;
  return vec4(r,g,b,a);
}

void main() {
  vec4 colorA;
  // border width between 1 - 0
  float width = abs(v_width) / (v_circleRadius * 2.0);
  float borderWidth = abs(v_width);
  vec2 circleCentre = vec2(0.5, 0.5);
  int shadowType= roundToInt(u_shadowType);
  bool isCircle = v_circleRadius > 0.0;
  bool hasRectangleBorder = roundToInt(v_circleRadius) < 0;
  bool hasCircleDashedBorder = v_width < 0.0;
  bool onlyDrawOnShadow = shadowType == 1;
  bool shadowCanvas = roundToInt(u_hitTest) == 1;
  vec2 textureCoords = v_texCoord.zw;
  // -2 flat shaded with anti aliasing
  // -1 flat shaded
  // 0..2 -> text
  // 3 -> fontIcons
  // 4...7 -> images
  int textureBank = roundToInt(v_textureBank.x);

// 
  colorA = v_colour / 255.0;
  // load the correct texture and read it
  if (textureBank >= 0) {
      vec4 texCoord;
      // set mipmap level bias to sharpen textures
      float bias = -2.0/3.0;
      float textBias = -0.8;
      //text 0
      if(textureBank == 0)  {
       texCoord = texture2D(u_image[0], v_texCoord.xy, textBias);
      //text 1
      } else if(textureBank == 1) {
        texCoord = texture2D(u_image[1], v_texCoord.xy, textBias);
      // text 2
      } else if(textureBank == 2) {
        texCoord = texture2D(u_image[2], v_texCoord.xy, textBias);
      // fontIcons 3 
      } else if(textureBank == 3) {
        texCoord = texture2D(u_image[3], v_texCoord.xy, bias);
      // image 4   
      } else if(textureBank == 4) {
        texCoord = texture2D(u_image[4], v_texCoord.xy, bias); 
      }
      float colourComponent = readColourComponentFromTexture(v_textureBank.y, texCoord);    
      colorA = getColourFromComponent(colourComponent, colorA);
  }
  gl_FragColor = colorA;
   
}